==> MT-1.1.tar.gz: MT-1.1/lib/MT.pm <==
# Copyright 2001 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: MT.pm,v 1.41 2001/10/18 16:58:37 btrott Exp $

package MT;
use strict;

use vars qw( $VERSION );
$VERSION = '1.1';

use MT::ConfigMgr;
use MT::Object;
use MT::Blog;
use MT::Util qw( start_end_day start_end_week start_end_month
                 archive_file_for );
use File::Spec;
use Fcntl qw( LOCK_EX );

use MT::ErrorHandler;
use base qw( MT::ErrorHandler );

sub version_slug {
    return <<SLUG;
Powered by Movable Type
Version $VERSION
http://www.movabletype.org/
SLUG
}

sub new {
    my $class = shift;
    my $mt = bless { }, $class;
    $mt->init(@_) or
        return $class->error($mt->errstr);
    $mt;
}

sub init {
    my $mt = shift;
    my %param = @_;
    my $cfg = $mt->{cfg} = MT::ConfigMgr->instance;
    my($cfg_file);
    unless ($cfg_file = $param{Config}) {
        for my $f (qw( mt.cfg )) {
            $cfg_file = $f, last if -r $f;
        }
    }
    if ($cfg_file) {
        $cfg->read_config($cfg_file) or
            return $mt->error($cfg->errstr);
    }
    MT::Object->set_driver($cfg->ObjectDriver);
    $mt;
}

sub rebuild {
    my $mt = shift;
    my %param = @_;
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $param{BlogID};
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                               MT::Blog->errstr);
    }
    my $at = $blog->archive_type || '';
    my @at = split /,/, $at;
    if (my $set_at = $param{ArchiveType}) {
        my %at = map { $_ => 1 } @at;
        return $mt->error("Archive type '$set_at' is not a chosen archive type")
            unless $at{$set_at};
        @at = ($set_at);
    }
    if (@at) {
        require MT::Entry;
        my $iter = MT::Entry->load_iter({ blog_id => $blog->id,
                                          status => MT::Entry::RELEASE() });
        my %done;
        my $cb = $param{EntryCallback};
        while (my $entry = $iter->()) {
            $cb->($entry) if $cb;
            for my $at (@at) {
                my $file = archive_file_for($entry, $blog, $at);
                next if $done{$file};
                $mt->_rebuild_entry_archive_type( Entry => $entry,
                                                  Blog => $blog,
                                                  ArchiveType => $at )
                    or return;
                $done{$file}++;
            }
        }
    }
    unless ($param{NoIndexes}) {
        $mt->rebuild_indexes( Blog => $blog ) or return;
    }
    1;
}

sub rebuild_entry {
    my $mt = shift;
    my %param = @_;
    my $entry = $param{Entry} or
        return $mt->error("Parameter 'Entry' is required");
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $entry->blog_id;
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    my $at = $blog->archive_type;
    return 1 if !$at || $at eq 'None';
    my @at = split /,/, $at;
    for my $at (@at) {
        $mt->_rebuild_entry_archive_type( Entry => $entry, Blog => $blog,
                                          ArchiveType => $at ) or
            return;
    }
    1;
}

sub _rebuild_entry_archive_type {
    my $mt = shift;
    my %param = @_;
    my $at = $param{ArchiveType} or
        return $mt->error("Parameter 'ArchiveType' is required");
    return 1 if $at eq 'None';
    my $entry = $param{Entry} or
        return $mt->error("Parameter 'Entry' is required");
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $entry->blog_id;
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    require MT::Template;
    require MT::Template::Context;
    my($tmpl, %cond);
    my $ctx = MT::Template::Context->new;
    my $file = File::Spec->catfile($blog->archive_path,
        archive_file_for($entry, $blog, $at));
    ## Untaint. We have to assume that we can trust the user's setting of
    ## the archive_path, and nothing else is based on user input.
    ($file) = $file =~ /(.+)/s;
    if ($at eq 'Individual') {
        $ctx->stash('entry', $entry);
        $ctx->{current_timestamp} = $entry->created_on;
        $cond{EntryIfAllowComments} = $entry->allow_comments;
        $cond{EntryIfExtended} = $entry->text_more ? 1 : 0;
        $tmpl = MT::Template->load({ type => 'individual',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Daily') {
        my($start, $end) = start_end_day($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Weekly') {
        my($start, $end) = start_end_week($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Monthly') {
        my($start, $end) = start_end_month($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Category') {
        if (my $cat_id = $entry->category_id) {
            require MT::Category;
            my $cat = MT::Category->load($entry->category_id);
            $ctx->stash('archive_category', $cat);
        }
        my @entries = MT::Entry->load({ category_id => $entry->category_id,
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'category',
                                     blog_id => $blog->id });
    }
    return $mt->error("You selected the archive type '$at', but you did not " .
                      "define a template for this archive type. Note that " .
                      "the default templates are currently only installed " .
                      "in the first blog that Movable Type ships with; they " .
                      "are not installed automatically with blogs that you " .
                      "create. To load those templates into your system, " .
                      "visit " .
                      "http://www.movabletype.org/default-templates.html .")
        unless $tmpl;
    my $html = $tmpl->build($ctx, \%cond) or
        return $mt->error("Building entry failed: " . $tmpl->errstr);
    local *FH;
    my $created = -e $file ? 0 : 1;
    open FH, ">$file" or
        return $mt->error("Opening '$file' failed: $!; did you set your " .
                          "Local Archive Path, and is the archive directory " .
                          "created and set to the correct permissions (777)?");
    flock FH, LOCK_EX;
    seek FH, 0, 0;
    truncate FH, 0;
    print FH $html;
    close FH;
    if ($created) {
        chmod 0666, $file or
            return $mt->error("Can't chmod 0666 '$file': $!");
    }
    1;
}
sub rebuild_indexes {
    my $mt = shift;
    my %param = @_;
    require MT::Template;
    require MT::Template::Context;
    require MT::Entry;
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $param{BlogID};
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    my $iter = MT::Template->load_iter({ type => 'index',
        blog_id => $blog->id });
    local *FH;
    my $site_root = $blog->site_path;
    while (my $tmpl = $iter->()) {
        my $ctx = MT::Template::Context->new;
        my $html = $tmpl->build($ctx);
        return $mt->error( $tmpl->errstr ) unless defined $html;
        my $index = File::Spec->catfile($site_root, $tmpl->outfile);
        ## Untaint. We have to assume that we can trust the user's setting of
        ## the site_path and the template outfile.
        ($index) = $index =~ /(.+)/s;
        my $created = -e $index ? 0 : 1;
        open FH, ">$index" or
            return $mt->error("Opening '$index' failed: $!");
        flock FH, LOCK_EX;
        seek FH, 0, 0;
        truncate FH, 0;
        print FH $html;
        close FH;
        if ($created) {
            chmod 0666, $index or
                return $mt->error("Can't chmod 0666 '$index': $!");
        }
    }
    1;
}

1;

==> MT-1.4.tar.gz: MT-1.4/lib/MT.pm <==
# Copyright 2001, 2002 Benjamin Trott. This code cannot be redistributed without
# permission from www.movabletype.org.
#
# $Id: MT.pm,v 1.56 2002/01/05 22:47:05 btrott Exp $

package MT;
use strict;

use vars qw( $VERSION );
$VERSION = '1.4';

use MT::ConfigMgr;
use MT::Object;
use MT::Blog;
use MT::Util qw( start_end_day start_end_week start_end_month
                 archive_file_for );
use File::Spec;
use Fcntl qw( LOCK_EX );

use MT::ErrorHandler;
@MT::ISA = qw( MT::ErrorHandler );

sub version_slug {
    return <<SLUG;
Powered by Movable Type
Version $VERSION
http://www.movabletype.org/
SLUG
}

sub new {
    my $class = shift;
    my $mt = bless { }, $class;
    $mt->init(@_) or
        return $class->error($mt->errstr);
    $mt;
}

sub init {
    my $mt = shift;
    my %param = @_;
    my $cfg = $mt->{cfg} = MT::ConfigMgr->instance;
    my($cfg_file);
    unless ($cfg_file = $param{Config}) {
        for my $f (qw( mt.cfg )) {
            $cfg_file = $f, last if -r $f;
        }
    }
    if ($cfg_file) {
        $cfg->read_config($cfg_file) or
            return $mt->error($cfg->errstr);
    }
    MT::Object->set_driver($cfg->ObjectDriver);
    $mt;
}

sub rebuild {
    my $mt = shift;
    my %param = @_;
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $param{BlogID};
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                               MT::Blog->errstr);
    }
    my $at = $blog->archive_type || '';
    my @at = split /,/, $at;
    if (my $set_at = $param{ArchiveType}) {
        my %at = map { $_ => 1 } @at;
        return $mt->error("Archive type '$set_at' is not a chosen archive type")
            unless $at{$set_at};
        @at = ($set_at);
    }
    if (@at) {
        require MT::Entry;
        my $iter = MT::Entry->load_iter({ blog_id => $blog->id,
                                          status => MT::Entry::RELEASE() });
        my %done;
        my $cb = $param{EntryCallback};
        while (my $entry = $iter->()) {
            $cb->($entry) if $cb;
            for my $at (@at) {
                my $file = archive_file_for($entry, $blog, $at);
                next if $done{$file};
                $mt->_rebuild_entry_archive_type( Entry => $entry,
                                                  Blog => $blog,
                                                  ArchiveType => $at )
                    or return;
                $done{$file}++;
            }
        }
    }
    unless ($param{NoIndexes}) {
        $mt->rebuild_indexes( Blog => $blog ) or return;
    }
    1;
}

sub rebuild_entry {
    my $mt = shift;
    my %param = @_;
    my $entry = $param{Entry} or
        return $mt->error("Parameter 'Entry' is required");
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $entry->blog_id;
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    my $at = $blog->archive_type;
    return 1 if !$at || $at eq 'None';
    my @at = split /,/, $at;
    for my $at (@at) {
        $mt->_rebuild_entry_archive_type( Entry => $entry, Blog => $blog,
                                          ArchiveType => $at ) or
            return;
    }
    1;
}

sub _rebuild_entry_archive_type {
    my $mt = shift;
    my %param = @_;
    my $at = $param{ArchiveType} or
        return $mt->error("Parameter 'ArchiveType' is required");
    return 1 if $at eq 'None';
    my $entry = $param{Entry} or
        return $mt->error("Parameter 'Entry' is required");
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $entry->blog_id;
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    require MT::Template;
    require MT::Template::Context;
    my($tmpl, %cond);
    my $ctx = MT::Template::Context->new;
    my $arch_root = $blog->archive_path;
    return $mt->error("You did not set your Local Archive Path")
        unless $arch_root;
    my $file = File::Spec->catfile($arch_root,
        archive_file_for($entry, $blog, $at));
    ## Untaint. We have to assume that we can trust the user's setting of
    ## the archive_path, and nothing else is based on user input.
    ($file) = $file =~ /(.+)/s;
    if ($at eq 'Individual') {
        $ctx->stash('entry', $entry);
        $ctx->{current_timestamp} = $entry->created_on;
        $cond{EntryIfAllowComments} = $entry->allow_comments;
        $cond{EntryIfExtended} = $entry->text_more ? 1 : 0;
        $tmpl = MT::Template->load({ type => 'individual',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Daily') {
        my($start, $end) = start_end_day($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Weekly') {
        my($start, $end) = start_end_week($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Monthly') {
        my($start, $end) = start_end_month($entry->created_on, $blog);
        $ctx->{current_timestamp} = $start;
        my @entries = MT::Entry->load({ created_on => [ $start, $end ],
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() },
                                      { range => { created_on => 1 } });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'archive',
                                     blog_id => $blog->id });
    } elsif ($at eq 'Category') {
        if (my $cat_id = $entry->category_id) {
            require MT::Category;
            my $cat = MT::Category->load($entry->category_id);
            $ctx->stash('archive_category', $cat);
        }
        my @entries = MT::Entry->load({ category_id => $entry->category_id,
                                        blog_id => $blog->id,
                                        status => MT::Entry::RELEASE() });
        $ctx->stash('entries', \@entries);
        $tmpl = MT::Template->load({ type => 'category',
                                     blog_id => $blog->id });
    }
    return $mt->error("You selected the archive type '$at', but you did not " .
                      "define a template for this archive type.")
        unless $tmpl;
    $ctx->{current_archive_type} = $at;
    my $html = $tmpl->build($ctx, \%cond) or
        return $mt->error("Building entry failed: " . $tmpl->errstr);

    ## Determine if we need to build directory structure, and build it
    ## if we do. DirUmask determines directory permissions.
    my($vol, $path, $fname) = File::Spec->splitpath($file);
    unless (-e $path) {
        require File::Path;
        my $umask = oct $mt->{cfg}->DirUmask;
        my $old = umask($umask);
        File::Path::mkpath([$path], 0, 0777);
        umask($old);
    }

    ## By default we write all data to temp files, then rename the temp
    ## files to the real files (an atomic operation). Some users don't
    ## like this (requires too liberal directory permissions). So we
    ## have a config option to turn it off (NoTempFiles).
    my $use_temp_files = !$mt->{cfg}->NoTempFiles;
    my $temp_file = $use_temp_files ? "$file.new" : $file;
    my $umask = oct $mt->{cfg}->HTMLUmask;
    my $old = umask($umask);
    local *FH;
    open FH, ">$temp_file" or
        return $mt->error("Opening '$temp_file' failed: $!; did you set your " .
                          "Local Archive Path, and is the archive directory " .
                          "created and set to the correct permissions (777)?");
    flock FH, LOCK_EX;
    seek FH, 0, 0;
    truncate FH, 0;
    print FH $html;
    close FH;
    if ($use_temp_files) {
        rename $temp_file, $file or
            return $mt->error("Renaming tempfile '$temp_file' failed: $!");
    }
    umask($old);
    1;
}
sub rebuild_indexes {
    my $mt = shift;
    my %param = @_;
    require MT::Template;
    require MT::Template::Context;
    require MT::Entry;
    my $blog;
    unless ($blog = $param{Blog}) {
        my $blog_id = $param{BlogID};
        $blog = MT::Blog->load($blog_id) or
            return $mt->error("Load of blog '$blog_id' failed: " .
                                 MT::Blog->errstr);
    }
    my $iter = MT::Template->load_iter({ type => 'index',
        blog_id => $blog->id });
    local *FH;
    my $site_root = $blog->site_path;
    return $mt->error("You did not set your Local Site Path")
        unless $site_root;
    while (my $tmpl = $iter->()) {
        my $ctx = MT::Template::Context->new;
        my $html = $tmpl->build($ctx);
        return $mt->error( $tmpl->errstr ) unless defined $html;
        my $index = File::Spec->catfile($site_root, $tmpl->outfile);
        ## Untaint. We have to assume that we can trust the user's setting of
        ## the site_path and the template outfile.
        ($index) = $index =~ /(.+)/s;
        my $use_temp_files = !$mt->{cfg}->NoTempFiles;
        my $temp_file = $use_temp_files ? "$index.new" : $index;
        my $umask = oct $mt->{cfg}->HTMLUmask;
        my $old = umask($umask);
        open FH, ">$temp_file" or
            return $mt->error("Opening '$temp_file' failed: $!");
        flock FH, LOCK_EX;
        seek FH, 0, 0;
        truncate FH, 0;
        print FH $html;
        close FH;
        if ($use_temp_files) {
            rename $temp_file, $index or
                return $mt->error("Renaming tempfile '$temp_file' failed: $!");
        }
        umask($old);
    }
    1;
}

1;
